#include "DarkGDK.h"

#include "config.h"

#include "app/AppKernel.h"
#include "render/RenderKernel.h"
#include "logic/LogicKernel.h"
#include "data/DataKernel.h"
#include "network/NetworkKernel.h"
#include "core/CoreKernel.h"

#include "core/system/System.h"

using Core::System;

void DarkGDK ( void )
{
  Log log;

  LOG_INFO << "START MAIN\n";


  Core::CoreKernel        coreKernel;
  Network::NetworkKernel  networkKernel;
  Data::DataKernel        dataKernel    ( coreKernel, networkKernel );
  Logic::LogicKernel      logicKernel   ( dataKernel );
  Render::RenderKernel    renderKernel  ( dataKernel );
  App::AppKernel          appKernel     ( renderKernel, logicKernel );

  int lastTime = System::getTimer();

  LOG_INFO << "main::start loop( " << lastTime << " )\n";
  LOG_FLUSH;

  while ( LoopGDK ( ) )
	{
    const int   currTime = System::getTimer();
    const float deltaTime = static_cast<float>( currTime - lastTime ) / 1000.0f;
    lastTime = currTime;

    appKernel.update ( deltaTime );

    if ( dbEscapeKey ( ) )
    {
			break;
    }

    Sleep(10);
	}
	
  LOG_INFO << "main::end loop( " << lastTime << " )\n";
  LOG_INFO << "END MAIN\n";
  LOG_FLUSH;
}