#include "Config.h"

#include <fstream>

namespace po = boost::program_options;

boost::program_options::variables_map Config::s_vm;

Config::Config(void)
{
  po::options_description configs("all configs");
  configs.add_options()
      ("display.width",           po::value<unsigned>(),    "display width")
      ("display.height",          po::value<unsigned>(),    "display height")
      ("display.depth",           po::value<unsigned>(),    "display depth")

      ("sprite.tileSize",         po::value<unsigned>(),    "")
      ("sprite.bombermanX",       po::value<unsigned>(),    "")
      ("sprite.bombermanY",       po::value<unsigned>(),    "")
      ("sprite.maxCount",         po::value<unsigned>(),    "")

      ("network.defaultPort",     po::value<unsigned>(),    "default port")
      ("network.maxConnections",  po::value<unsigned>(),    "max connections")
      ("network.broadcastDelay",  po::value<float>(),       "delay between broadcasts")

      ("level.path",              po::value<std::string>(), "")

      ("logging.file",            po::value<std::string>(), "")
      ("logging.level",           po::value<unsigned>(),    "")

      ("input_1.up",              po::value<unsigned>(),    "")
      ("input_1.down",            po::value<unsigned>(),    "")
      ("input_1.left",            po::value<unsigned>(),    "")
      ("input_1.right",           po::value<unsigned>(),    "")
      ("input_1.action_1",        po::value<unsigned>(),    "")
      ("input_1.action_2",        po::value<unsigned>(),    "")
      ("input_1.no",              po::value<unsigned>(),    "")
      ("input_1.yes",             po::value<unsigned>(),    "")
      ("input_1.escape",          po::value<unsigned>(),    "")
      ("input_1.enter",           po::value<unsigned>(),    "")
      ("input_1.stats",           po::value<unsigned>(),    "")
  ;

  po::options_description config_file_options;
  config_file_options.add(configs);

  std::ifstream configFile("../data/config.cfg");
  if( configFile )
  {
    try
    {
      LOG_INFO << "Config::Config() - parse config file\n";
      po::store( po::parse_config_file( configFile, config_file_options ), s_vm );
      po::notify( s_vm );    
    }
    catch(std::exception& e)
    {
        LOG_ERROR << "Config::Config() - while parsing (" << e.what() << " )\n";
    }    
  }
  else
  {
    LOG_ERROR << "Config::Config() - cannot open config file\n";
  }
}

Config::~Config(void)
{
}

Config config;