#include "Input.h"

#include "Config.h"
#include "DarkGDK.h"

#include "InputTable.h"

using namespace Core;


Input::Input(void)
{
}

Input::~Input(void)
{
}

void interpretCommandStates( CommandStates& _commandStates )
{
  //one movement released
  if(
      _commandStates[ Input::Command::UP ]    == CommandState::RELEASED ||
      _commandStates[ Input::Command::DOWN ]  == CommandState::RELEASED ||
      _commandStates[ Input::Command::LEFT ]  == CommandState::RELEASED ||
      _commandStates[ Input::Command::RIGHT ] == CommandState::RELEASED
    )
  {
  //no movement -> STOP
    if(
        ( _commandStates[ Input::Command::UP ]    == CommandState::INACTIVE || _commandStates[ Input::Command::UP ]    == CommandState::RELEASED ) &&
        ( _commandStates[ Input::Command::DOWN ]  == CommandState::INACTIVE || _commandStates[ Input::Command::DOWN ]  == CommandState::RELEASED ) &&
        ( _commandStates[ Input::Command::LEFT ]  == CommandState::INACTIVE || _commandStates[ Input::Command::LEFT ]  == CommandState::RELEASED ) &&
        ( _commandStates[ Input::Command::RIGHT ] == CommandState::INACTIVE || _commandStates[ Input::Command::RIGHT ] == CommandState::RELEASED )
      )
    {
      _commandStates[ Input::Command::STOP ] = CommandState::PRESSED;
    }
    else
    {
      //check, if one movement is still active -> trigger again
      if( _commandStates[ Input::Command::UP ] == CommandState::ACTIVE )
      {
        _commandStates[ Input::Command::UP ] = CommandState::PRESSED;
      }
      else if( _commandStates[ Input::Command::DOWN ] == CommandState::ACTIVE )
      {
        _commandStates[ Input::Command::DOWN ] = CommandState::PRESSED;
      }
      else if( _commandStates[ Input::Command::LEFT ] == CommandState::ACTIVE )
      {
        _commandStates[ Input::Command::LEFT ] = CommandState::PRESSED;
      }
      else if( _commandStates[ Input::Command::RIGHT ] == CommandState::ACTIVE )
      {
        _commandStates[ Input::Command::RIGHT ] = CommandState::PRESSED;
      }
    }
  }

}

InputTable& inputTable()
{
  static InputTable s_inputTable;
  return s_inputTable;
}


void
Input::update( const float _secs )
{
  CommandStates commandStates;

  inputTable().getCommandStates( commandStates );

  interpretCommandStates( commandStates );

  for( CommandStates::const_iterator it = commandStates.begin(); it != commandStates.end(); it++ )
  {
    if( (*it).second == CommandState::PRESSED )
    {
      eventCommand().notify( (*it).first );
    }
  }

}


////////////////////
// SIGNALS

ObserverSubject< const Input::Command >&
Input::eventCommand()
{
  static ObserverSubject< const Command >  s_eventCommand;
  return s_eventCommand;
}
// SIGNALS
////////////////////

bool
Input::isKeyPressed( int _scanCode )
{
  return ( dbKeyState( _scanCode ) != 0 );
}