#ifndef CORE_OBSERVER_H_
#define CORE_OBSERVER_H_

#include <deque>
#include <algorithm>

namespace Core
{
  template< typename T>
  class Observer
  {
    public:
      virtual void notify( T _t ) = 0 ;
  } ;

  template< typename T>
  class ObserverSubject
  {
  private:
    typedef std::deque< Observer<T>* > Observers ;
    Observers m_observers ;

  public:
    bool add( Observer<T>* _observer )
    {
      Observers::const_iterator foundObserver = std::find( m_observers.begin(), m_observers.end(), _observer ) ;
      if( foundObserver == m_observers.end() )
      {
        m_observers.push_back( _observer ) ;
        return true ;
      }
      else
      {
        return false ;
      }
    }

    bool remove( Observer<T>* _observer )
    {
      Observers::iterator foundObserver = std::find( m_observers.begin(), m_observers.end(), _observer ) ;
      if( foundObserver != m_observers.end() )
      {
        m_observers.erase( foundObserver ) ;
        return true ;
      }
      else
      {
        return false ;
      }
    }

    void notify( T _t )
    {
      Observers::const_iterator allObservers = m_observers.begin() ;
      while( allObservers != this->m_observers.end() )
      {
        ( *allObservers )->notify( _t ) ;
        allObservers++;
      }
    }

    bool isUsed()
    {
      return m_observers.size() > 0;
    }
  };


}; //Core

#endif //CORE_OBSERVER_H_
