#include "World.h"

#include <vector>

#include "data/world/Tiles.h"
#include "data/entity/Creature.h"

#include <fstream>
#include <vector>

namespace Data
{
  Tiles     s_tiles;
  
  typedef   std::vector< Creature* const > Creatures;
  Creatures s_creatures;



class CreateCreatureObserver : public Core::Observer< Data::Creature* const >
{
  public:
    CreateCreatureObserver()
    {
      Creature::eventCreateCreature().add(this);
    }

    virtual void notify( Data::Creature* const _data ) 
    {
      s_creatures.push_back( _data );
    }
} creatureCreateObserver;

class DestroyCreatureObserver : public Core::Observer< Data::Creature* const >
{
  public:
    DestroyCreatureObserver()
    {
      Creature::eventDestroyCreature().add(this);
    }

    virtual void notify( Data::Creature* const _data ) 
    {
      s_creatures.erase( find( s_creatures.begin(), s_creatures.end(), _data) );
    }
} creatureDestroyObserver;


}



using namespace Data;

World::World(void)
{
  init();
}

World::~World(void)
{
  deinit();
}

void
World::init()
{
  s_tiles.init();
  import( "oldschool.lvl" );
}

void 
World::deinit()
{
  s_tiles.deinit();
}


const Tiles&
World::tiles()
{
  return s_tiles;
}

void
World::update( const float _secs )
{
  s_tiles.update( _secs);

  for( Creatures::iterator it = s_creatures.begin(); it != s_creatures.end(); it++ )
  {
    (*it)->update(_secs);
  }
}

typedef std::vector<std::string> Lines;

bool
World::import( const std::string& _file )
{
  // PATH
  std::string path = Config::get<std::string>("level.path");
  path += _file;
  
  ////////
  // FILE
  Lines lines;
  {
    std::ifstream fileStream ( path.c_str() );

    if(! fileStream.is_open() ) return false;


    // READ IN
    while (! fileStream.eof() )
    {
      std::string line;
      getline( fileStream, line );
      lines.push_back( line );
    }

    fileStream.close();
  }
  // FILE
  ////////

  //////////
  // PARSER
  if(! lines.size() ) return false;

  // SANITY
  unsigned lineLength;
  {
    Lines::const_iterator it = lines.begin();
    lineLength = (*it).length();

    for( ; it != lines.end(); it++ )
    {
      if( (*it).length() != lineLength ) return false;
    }
  }

  // INIT
  s_tiles.resize( lineLength + 2, lines.size() + 2 );

  // IMPORT
  {
    unsigned y(1);

    for( Lines::const_iterator itY = lines.begin(); itY != lines.end(); itY++ )
    {
      unsigned x(1);
      for( std::string::const_iterator itX = (*itY).begin(); itX != (*itY).end(); itX++ )
      {
        Tiles::Tile tile;
        switch( *itX )
        {
          case '*' : tile = Tiles::Tile::SOLID; break;
          case '#' : tile = Tiles::Tile::ROCK;  break;
          case '1' :
          case '2' :
          case '3' :
          case '4' :
          case '5' :
          case '6' :
          case '7' :
          case '8' :
          case '_' : tile = Tiles::Tile::FREE; break;
          default  : return false;
        }

        s_tiles.setTile( Unsigned2d( x, y ), tile );
        x++;
      }
      y++;
    }
  }
  // PARSER
  //////////

  return true;
}


