#pragma once

#include <map>

#include "Replica.h"

namespace Network
{

typedef unsigned                                          Type;
typedef Replica* (*FactoryCreateFunction)();
typedef std::map< Type, FactoryCreateFunction >           Factories;

class ReplicaFactory
{
  private:
    static Factories& factories();

  public:
    ReplicaFactory();
    ~ReplicaFactory();

    static void 	  registerFactory( const Type& _type, FactoryCreateFunction _factory);

    static Replica* create  ( const Type _type );
};

template <class Type>
class ReplicaFactoryRegister
{
	public:
		ReplicaFactoryRegister()
		{
			ReplicaFactory::registerFactory( Type::staticType(), &Type::create );
		}
};

}; //Network

